/*
 *	asm.c - assemble
 */
#include "asm.h"

static TOKEN	tok;
static int	    errorcount;
static OPS	    ops[2];

static char	*errmsg[] = {
	"Unknown error",
	"Syntax error",
	"Operand error",
	"Too many instructions",
	"Label name redefined",
	"Not enough memory",
	"Label not defined",
	"Interger overflow",
	"Divide by zero",
};
#define MAX_ERROR	(sizeof(errmsg) / sizeof(errmsg[0]))

/*
 *	doError
 */
static void doError(int eno)
{
	char	*s;

	if (eno < 0 || eno >= MAX_ERROR)
		eno = E_UNKNOWN;
	s = errmsg[eno];
	fprintf(stderr, "%s: %s: Line %d: %s.\n",
		GetCmdName(), GetSrcName(), GetCurLineNo(), s);
	errorcount++;
	CancelLine();
}

#define OBJCOL	12
#define OBJLEN	4
static char	obuf[OBJCOL+1];

/*
 *	dolist
 */
static void dolist(FILE *lfp, int saddr)
{
	char	*op, *ibp;
	int 	eaddr;
	BOOL	first;
	int		i, size, count, r;

	if (!lfp)
		return;

	ibp = GetCurLine();
	if (!IsObjectDirty())
	{
		fprintf(lfp, "%*c%s", OBJCOL, ' ', ibp);
		return;
	}

	eaddr = GetCurAddr();
	size = eaddr - saddr;
	first = TRUE;
	while (size > 0)
	{
		op = obuf;
		if (first)
			op += sprintf(op, "%02x:", saddr);
		else
			op += sprintf(op, "   ");
		count = min(size, OBJLEN);
		for (i = 0; i < count; i++)
			op += sprintf(op, "%1x ", GetNibble(saddr++));
		r = max(OBJCOL - (op - obuf), 0);
		if (first)
			fprintf(lfp, "%s%*c%s", obuf, r, ' ', ibp);
		else
			fprintf(lfp, "%s\n", obuf);
		first = FALSE;
		size -= count;
	}
}

/*
 *	DoAsm
 */
int DoAsm(BOOL bdump, FILE *lfp)
{
	int		t, e;
	INST	*ip;
	int		saddr;

	errorcount = 0;
	ip = NULL;
	for (;;)
	{
		ClearCurLabel();
		ClearObjectDirty();
		saddr = GetCurAddr();
		t = GetToken(&tok);
		if (t == T_NAME && FindInst(tok.name) == NULL)
		{
			if ((e = DefineLabel(tok.name)) != 0)
			{
				doError(e);
				continue;
			}
			t = GetToken(&tok);
			if (t == ':')
				t = GetToken(&tok);
		}
		if (t == T_EOF)
			break;
        else if (t == T_EOL)
			goto next;
		else if (t != T_NAME)
		{
			doError(E_SYNTAX);
			continue;
		}
		if ((ip = FindInst(tok.name)) == NULL)
		{
			doError(E_SYNTAX);
			continue;
		}
		if (ip->op1)
		{
			if ((e = (*ip->op1)(&ops[0])) != 0)
			{
				doError(e);
				continue;
			}
		}
		if (ip->op2)
		{
			if ((t = GetToken(&tok)) != ',')
			{
				doError(E_SYNTAX);
				continue;
			}
			if ((e = (*ip->op2)(&ops[1])) != 0)
			{
				doError(e);
				continue;
			}
		}
		if ((e = (*ip->disp)(ip, ops)) != 0)
		{
			doError(e);
			continue;
		}
		t = GetToken(&tok);
		if (t != T_EOL)
		{
			doError(E_SYNTAX);
			continue;
		}
	next:
		dolist(lfp, saddr);
		if (ip && stricmp(ip->name, "end") == 0)
			break;
	}
	return errorcount;
}

// vi: ts=4 sw=4

