/*
 *	asm.h - header for gmc4asm
 */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <limits.h>

#include <windows.h>

/*
 *  constants
 */

#define NIBBLE      15

#define OBJ_ADDR    0x00
#define OBJ_SIZE    0x60

/*
 *	token
 */

#define MAX_NAME	80

typedef union _TOKEN {
	char	name[MAX_NAME];
	int 	value;
} TOKEN;

// token types
#define T_EOF		(-1)
#define T_ERROR		0xEE00
#define T_EOL		0xF000
#define T_CHAR		0x0000
#define T_NAME		0x0100
#define T_VALUE		0x0200

/*
 *	error
 */

#define E_UNKNOWN	    0	// Not an error
#define E_SYNTAX	    1
#define E_OPERAND	    2
#define E_NOROOM	    3
#define E_REDEFLABEL	4
#define E_NOMEM		    5
#define E_NODEFLABEL	6
#define E_INTOVER	    7
#define E_DIVZERO	    8

/*
 *	instruction
 */

typedef union _OPS {
	int 	index;
	int 	value;
} OPS;

typedef struct _INST {
	char	*name;
	int	    (*disp)(struct _INST *ip, OPS *op);
	int 	code;
	int	    (*op1)(OPS *op);
	int	    (*op2)(OPS *op);
} INST;

extern INST	InstTab[];

/*
 *	label
 */

typedef struct _LABEL {
	struct _LABEL	*next;
	char		    *name;
	int 		    value;
} LABEL;

/*
 *	Export functions
 */

/* main.c */
char *GetCmdName();
char *GetSrcName();
int GetPass();

/* asm.c */
int DoAsm(BOOL bdump, FILE *lfp);

/* token.c */
void InitToken(FILE *fp);
int GetCurLineNo();
char *GetCurLine();
int GetToken(TOKEN *tp);
void UnGetToken();
void CancelLine();

/* inst.c */
INST *FindInst(char *name);

/* obj.c */
BOOL InitObject();
BOOL DumpObject(FILE *fp);
BOOL WriteObject(FILE *fp);
int PutNibble(int n);
int PutNibble(int n);
int GetNibble(int addr);
int GetCurAddr();
int SetCurAddr(int addr);
int CommitObject(int size);

/* label.c */
int DefineLabel(char *name);
void ClearCurLabel();
int GetLabelValue(char *name, int *wp, BOOL force);
int SetCurLabelValue(int value);
BOOL WriteMap(FILE *fp);

/* expr.c */
int GetExpr(int *vp, BOOL force);

// vi: ts=4 sw=4

