/*
 *	expr.c - expressions
 */
#include "asm.h"

static TOKEN	tok;

/*
 *	GetValue
 */
static int GetValue(int *vp, BOOL force)
{
	int		t;

	switch (t = GetToken(&tok))
    {
	default:
		UnGetToken();
		return E_OPERAND;
	case T_VALUE:
		*vp = tok.value;
		return 0;
	case T_NAME:
		return GetLabelValue(tok.name, vp, force);
	case '$':
		*vp = GetCurAddr();
		return 0;
	}
}

/*
 *	GetItem
 */
static int GetItem(int *vp, BOOL force)
{
	int		t, e;

	if ((e = GetValue(vp, force)) == 0)
		return 0;
	t = GetToken(&tok);
	if (t == '-')
    {
		if ((e = GetValue(vp, force)) != 0)
			return e;
		*vp = -(*vp);
		return 0;
	}
    else if (t == '(')
    {
		if ((e = GetExpr(vp, force)) != 0)
			return e;
		t = GetToken(&tok);
		if (t != ')')
			return E_OPERAND;
		return 0;
	}
    else
    {
		UnGetToken();
		return e;
	}
}

/*
 *	myMulDiv
 */
static int myMulDiv(int *vp, BOOL force)
{
	int		t, e;
	int		v1;

	if ((e = GetItem(vp, force)) != 0)
		return e;
	for (;;)
    {
		v1 = *vp;
		t = GetToken(&tok);
		if (t != '*' && t != '/' && t != '%')
        {
			UnGetToken();
			break;
		}
		if ((e = GetItem(vp, force)) != 0)
			return e;
		if (t == '*')
			*vp = v1 * *vp;
		else
        {
			if (*vp == 0)
				return E_DIVZERO;
			if (t == '/')
				*vp = v1 / *vp;
			else
				*vp = v1 % *vp;
		}
	}
	return 0;
}

/*
 *	GetExpr
 */
int GetExpr(int *vp, BOOL force)
{
	int		t, e;
	int		v1;

	if ((e = myMulDiv(vp, force)) != 0)
		return e;
	for (;;)
    {
		v1 = *vp;
		t = GetToken(&tok);
		if (t != '+' && t != '-')
        {
			UnGetToken();
			break;
		}
		if ((e = myMulDiv(vp, force)) != 0)
			return e;
		if (t == '+')
			*vp = v1 + *vp;
		else
			*vp = v1 - *vp;
	}
	return 0;
}

// vi: ts=4 sw=4

