/*
 *	inst.c - instruction handler
 */
#include "asm.h"

static TOKEN	tok;

/* === Instructions === */

static int iNoOp(INST *ip, OPS *op)
{
	return PutNibble(ip->code);
}

static int iI4(INST *ip, OPS *op)
{
	int		r;

	if ((r = PutNibble(ip->code)) != 0)
		return r;
	return PutNibble(op[0].value);
}

static int iI8(INST *ip, OPS *op)
{
	int		r;

	if ((r = PutNibble(ip->code)) != 0)
		return r;
	return PutNibble2(op[0].value);
}

/* === assembler instruction === */

static int iOrg(INST *ip, OPS *op)
{
	return SetCurAddr(op[0].value);
}

static int iEqu(INST *ip, OPS *op)
{
	return SetCurLabelValue(op[0].value);
}

static int iDn(INST *ip, OPS *op)
{
	int		t, e;
	int		v;

	for (;;)
    {
		if ((e = GetExpr(&v, FALSE)) != 0)
			return E_OPERAND;
		if ((short)v < -NIBBLE || (short)v > NIBBLE)
			return E_INTOVER;
		if ((e = PutNibble(v)) != 0)
			return e;
		t = GetToken(&tok);
		if (t != ',')
		{
			UnGetToken();
			break;
		}
	}
	return 0;
}

static int iDs(INST *ip, OPS *op)
{
	return CommitObject(op[0].value);
}

static int iEnd(INST *ip, OPS *op)
{
	return 0;
}

/* === operands === */

static int getindex(char *name, char **table)
{
	char		**tp;

	tp = table;
	while (*tp)
	{
		if (stricmp(name, *tp) == 0)
			return (int)(tp - table);
		tp++;
	}
	return -1;
}

static char	*tCals[] =
{
    "rsto", // 0
    "setr", // 1
    "rstr", // 2
    "inpt", // 3: not used for GMC-4
    "cmpl", // 4
    "chng", // 5
    "sift", // 6
    "ends", // 7
    "errs", // 8
    "shts", // 9
    "lons", // a
    "sund", // b
    "timr", // c
    "dspr", // d
    "dem-", // e
    "dem+", // f
    NULL,
};

static int oCal(OPS *op)
{
	int		t;

	if ((t = GetToken(&tok)) != T_NAME)
		return E_OPERAND;
	if ((op->index = getindex(tok.name, tCals)) < 0)
		return E_OPERAND;
    if (op->index == 3) // inpt: not supported on GMC-4
		return E_OPERAND;
	return 0;
}

static int oImm4(OPS *op)
{
	int		e;

	if ((e = GetExpr(&op->value, TRUE)) != 0)
		return e;
	if ((int)(op->value) < -NIBBLE || (int)(op->value) > NIBBLE)
		return E_INTOVER;
	else
		return 0;
}

static int oImm8(OPS *op)
{
	int		e;

	if ((e = GetExpr(&op->value, TRUE)) != 0)
		return e;
	if ((int)(op->value) < OBJ_ADDR || (int)(op->value) >= (OBJ_ADDR+OBJ_SIZE))
		return E_NOROOM;
	else
		return 0;
}

static int oAddr(OPS *op)
{
	int		e;

	if ((e = GetExpr(&op->value, FALSE)) != 0)
		return e;
	if ((int)(op->value) < OBJ_ADDR || (int)(op->value) > (OBJ_ADDR+OBJ_SIZE))
		return E_NOROOM;
	else
		return 0;
}

/* === instruction dispatch === */

/*
 *	Instruction table
 */

static INST	InstTab[] =
{

    //
    // original instructions
    //

	{ "ka", 	iNoOp,	0x0,	NULL,	NULL,	},
	{ "ao", 	iNoOp,	0x1,	NULL,	NULL,	},
	{ "ch", 	iNoOp,	0x2,	NULL,	NULL,	},
	{ "cy", 	iNoOp,	0x3,	NULL,	NULL,	},
	{ "am", 	iNoOp,	0x4,	NULL,	NULL,	},
	{ "ma", 	iNoOp,	0x5,	NULL,	NULL,	},
	{ "m+", 	iNoOp,	0x6,	NULL,	NULL,	},
	{ "m-", 	iNoOp,	0x7,	NULL,	NULL,	},
	{ "tia", 	iI4,	0x8,	oImm4,	NULL,	},
	{ "aia", 	iI4,	0x9,	oImm4,	NULL,	},
	{ "tiy", 	iI4,	0xa,	oImm4,	NULL,	},
	{ "aiy", 	iI4,	0xb,	oImm4,	NULL,	},
	{ "cia", 	iI4,	0xc,	oImm4,	NULL,	},
	{ "ciy", 	iI4,	0xd,	oImm4,	NULL,	},
	{ "cal", 	iI4,	0xe,	oCal,	NULL,	},
	{ "jump", 	iI8,	0xf,	oAddr,	NULL,	},

    //
    // extended instructions
    //

    // TODO:

    //
    // assemble control instructions
    //

	{ "org",	iOrg,	0,	oImm8,	NULL,	},
	{ "equ",	iEqu,	0,	oImm8,	NULL,	},
	{ "dn",		iDn,	0,	NULL,	NULL,	},
	{ "ds",		iDs,	0,	oImm8,	NULL,	},
	{ "end",	iEnd,	0,	NULL,	NULL,	},

	{ NULL },
};

/*
 *	FindInst
 */
INST *FindInst(char *name)
{
	INST		*ip;

	ip = InstTab;
	while (ip->name)
	{
		if (stricmp(name, ip->name) == 0)
			return ip;
		ip++;
	}
	return NULL;
}

// vi: ts=4 sw=4

