/*
 *	label.c - Label manager
 */
#include "asm.h"

static LABEL	*deflabel;
static LABEL	*curlabel;

/*
 *	FindLabel
 */
static LABEL *FindLabel(char *name)
{
	LABEL		*lp;

	for (lp = deflabel; lp; lp = lp->next)
    {
		if (strcmp(lp->name, name) == 0)
			return lp;
	}
	return NULL;
}

/*
 *	DefineLabel
 */
int DefineLabel(char *name)
{
	LABEL		*lp;

	if (GetPass() != 1)
    {
		if ((lp = FindLabel(name)) == NULL)
			return E_NODEFLABEL;
		curlabel = lp;
		return 0;
	}

	if ((lp = FindLabel(name)) != NULL)
		return E_REDEFLABEL;
	if ((lp = malloc(sizeof(*lp))) == NULL)
		return E_NOMEM;
	memset(lp, 0, sizeof(*lp));

	lp->next = deflabel;
	deflabel = lp;
	curlabel = lp;

	if ((lp->name = malloc(strlen(name) + 1)) == NULL)
		return E_NOMEM;
	strcpy(lp->name, name);
	lp->value = GetCurAddr();
	return 0;
}

/*
 *	ClearCurLabel
 */
void ClearCurLabel()
{
	curlabel = NULL;
}

/*
 *	GetLabelValue
 */
int GetLabelValue(char *name, int *wp, BOOL force)
{
	LABEL		*lp;

	if (!force && GetPass() == 1)
    {
		*wp = 0;
		return 0;
	}

	if ((lp = FindLabel(name)) == NULL)
		return E_NODEFLABEL;
	*wp = lp->value;
	return 0;
}

/*
 *	SetCurLabelValue
 */
int SetCurLabelValue(int value)
{
	if (curlabel == NULL)
		return E_SYNTAX;
	curlabel->value = value;
	return 0;
}

/*
 *	WriteMap
 */
BOOL WriteMap(FILE *fp)
{
	LABEL		*lp;

	for (lp = deflabel; lp; lp = lp->next)
    {
		if (fprintf(fp, "%-16s%04xh\n", lp->name, lp->value) < 0)
			return FALSE;
	}
	return TRUE;
}

// vi: ts=4 sw=4

