/*
 *	main.c - main procedures
 */
#include "asm.h"

static char	cmdname[] = "gmc4asm";
static char	*Usage[] =
{
	"GMC-4 Assembler - ver. 0.1",
	"Copyright (C) 2009 by yasuho",
	"",
	"Usage: gmc4asm [options] source-file",
    "Options:",
    "        -d: output dump format file (.dmp)",
    "        -s: generate simulator file (.fxp)",
	NULL
};

static BOOL	dflag = FALSE;
static BOOL	sflag = FALSE;

static char	*srcfile;
static char	basename[MAX_PATH];
static char *objfile;
static char	objname[MAX_PATH];
static char	objext[] = ".fxp";
static char *dmpfile;
static char	dmpname[MAX_PATH];
static char	dmpext[] = ".dmp";

static int	pass;

/*
 *	errorexit
 */
static void errorexit(int e)
{
	if (objfile) unlink(objfile);
	exit(1);
}

/*
 *	error
 */
static void error(char *s)
{
	fprintf(stderr, "%s: ", cmdname);
	fprintf(stderr, "%s\n", s);
	errorexit(1);
}

/*
 *	fatal
 */
static void fatal(char *s)
{
	fprintf(stderr, "%s: ", cmdname);
	perror(s);
	errorexit(1);
}

/*
 *	usage
 */
static void usage()
{
	char		**cpp;

	for (cpp = Usage; *cpp; cpp++)
		fprintf(stderr, "%s\n", *cpp);
	errorexit(1);
}

/*
 *	main
 */
main(int argc, char **argv)
{
	char	*cp, *sp, *bp;
	FILE	*sfp, *lfp, *ofp, *dfp;
	char	c;
	int		len;

    //
    // options
    //

	argc--, argv++;
	while (argc && **argv == '-')
    {
		cp = &argv[0][1];
		while ((c = *cp++) != '\0') switch (c)
        {
		default:
			usage();
		case 'd':
			dflag = TRUE;
			break;
		case 's':
			sflag = TRUE;
			break;
		}
		argc--, argv++;
	}
	if (argc == 0)
		usage();
	srcfile = *argv;

    //
	// build output filename
    //

    if (dflag || sflag)
    {
        len = strlen(srcfile);
        sp = &srcfile[len];
        while (len > 0)
        {
            len--;
            if (*--sp == '.')
                break;
        }
        if (len == 0)
            len = strlen(srcfile) - 1;
        strncpy(basename, srcfile, len);
        basename[len] = '\0';
	}

	if (dflag)
	{
        sprintf(dmpname, "%s%s", basename, dmpext);
		dmpfile = dmpname;
    }

	if (sflag)
	{
        sprintf(objname, "%s%s", basename, objext);
		objfile = objname;
    }

    //
	// assemble
    //

	for (pass = 1; pass <= 2; pass++)
    {
		// printf("=== pass %d ===\n", GetPass());

		// init
		if ((sfp = fopen(srcfile, "r")) == NULL)
			fatal(srcfile);
		InitToken(sfp);
		if (!InitObject())
			error("InitObject: failed.");

        lfp = (pass == 1) ? NULL : stdout;

		if (DoAsm(dflag, lfp))
			errorexit(1);

		fclose(sfp);
	}

    //
	// finishing
    //

    if (dflag)
    {
        if ((dfp = fopen(dmpfile, "wb")) == NULL)
            fatal(dmpfile);
        if (!DumpObject(dfp))
            error("DumpObject: failed.\n");
        fclose(dfp);
    }

    if (sflag)
    {
        if ((ofp = fopen(objfile, "wb")) == NULL)
            fatal(objfile);
        if (!WriteObject(ofp))
            error("WriteObject: failed.\n");
        fclose(ofp);
    }

	exit(0);
}

/*
 *	GetCmdName
 */
char *GetCmdName()
{
	return cmdname;
}

/*
 *	GetSrcName
 */
char *GetSrcName()
{
	return srcfile;
}

/*
 *	GetPass
 */
int GetPass()
{
	return pass;
}

// vi: ts=4 sw=4

