/*
 *	obj.c - object manager
 */
#include "asm.h"

static BYTE	*object;
static int	curoff;
static BOOL	dirty;

/*
 *	InitObject
 */
BOOL InitObject()
{
	if (object == NULL && (object = malloc(OBJ_SIZE)) == NULL)
		return FALSE;
	memset(object, 0xf, OBJ_SIZE);
	curoff = 0;
	return TRUE;
}

/*
 *	DumpObject
 */
BOOL DumpObject(FILE *fp)
{
	int		i;

	for (i = 0; i < OBJ_SIZE; i++)
	{
		if ((i % 16) == 0)
			fprintf(fp, "%02x:", OBJ_ADDR + i);

		fprintf(fp, " %x", GetNibble(OBJ_ADDR + i));

		if ((i % 16) == 15)
			fprintf(fp, "\r\n");
	}

	return TRUE;
}

/*
 *	WriteObject
 */
BOOL WriteObject(FILE *fp)
{
	int		i;

	fprintf(fp, "<?xml version=\"1.0\"?>\r\n");
	fprintf(fp, "<ArrayOfString xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
	fprintf(fp, " xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\r\n");

	for (i = 0; i < OBJ_SIZE; i++)
	{
		fprintf(fp, "  <string>%x</string>\r\n", GetNibble(OBJ_ADDR + i));
	}

	fprintf(fp, "</ArrayOfString>\r\n");

	return TRUE;
}

/*
 *	PutNibble
 */
int PutNibble(int n)
{
	if (curoff+1 > OBJ_SIZE)
		return E_NOROOM;
	object[curoff++] = n;
	dirty = TRUE;
	return 0;
}

/*
 *	PutNibble2
 */
int PutNibble2(int n)
{
	if (curoff+2 > OBJ_SIZE)
		return E_NOROOM;
	object[curoff++] = (n >> 4) & 0xf;
	object[curoff++] = (n & 0xf);
	dirty = TRUE;
	return 0;
}

/*
 *	GetNibble
 */
int GetNibble(int addr)
{
	if (addr < OBJ_ADDR || addr >= OBJ_ADDR + OBJ_SIZE)
		return 0;
	else
		return object[addr - OBJ_ADDR] & 0xf;
}

/*
 *	GetCurAddr
 */
int GetCurAddr()
{
	return OBJ_ADDR + curoff;
}

/*
 *	SetCurAddr
 */
int SetCurAddr(int addr)
{
	if (addr < OBJ_ADDR || addr >= OBJ_ADDR+OBJ_SIZE)
		return E_NOROOM;
	curoff = addr - OBJ_ADDR;
	return 0;
}

/*
 *	CommitObject
 */
int CommitObject(int size)
{
	if (curoff+size > OBJ_SIZE)
		return E_NOROOM;
	curoff += size;
	return 0;
}

/*
 *	ClearObjectDirty
 */
void ClearObjectDirty()
{
	dirty = FALSE;
}

/*
 *	IsObjectDirty
 */
BOOL IsObjectDirty()
{
	return dirty;
}

// vi: ts=4 sw=4

