/*
 *	token.c - token manager
 */
#include "asm.h"

static FILE	*in;
static int	curline;
static char	inbuf[128];
static char	*tokenp;
static char	*lasttokenp;

/*
 *	tohex
 */
static int tohex(char *s, int *wp)
{
	int		v;
	int		c;

	v = 0;
	for (;;)
    {
		c = toupper(*s);
		if (c == '\0')
			break;
		if (!isxdigit(c))
			return E_OPERAND;
		s++;
		v <<= 4;
		v += isdigit(c) ? (c - '0') : (c - 'A' + 10);
		if (v > USHRT_MAX)
			return E_INTOVER;
	}
	*wp = v;
	return 0;
}

/*
 *	toint
 */
static int toint(char *s, int *wp)
{
	int		v;
	int		c;

	v = 0;
	for (;;)
    {
		if ((c = *s++) == '\0')
			break;
		if (!isdigit(c))
			return E_OPERAND;
		v *= 10;
		v += (c - '0');
		if (v > USHRT_MAX)
			return E_INTOVER;
	}
	*wp = v;
	return 0;
}

/*
 *	InitToken
 */
void InitToken(FILE *fp)
{
	in = fp;
	curline = 0;
	tokenp = lasttokenp = inbuf;
	*tokenp = '\0';
}

/*
 *	GetCurLineNo
 */
int GetCurLineNo()
{
	return curline;
}

/*
 *	GetCurLine
 */
char *GetCurLine()
{
	return inbuf;
}

/*
 *	GetToken
 */
int GetToken(TOKEN *tp)
{
	int		c, e;
	char	*cp, *ep;
    int     len;

	if (feof(in))
		return T_EOF;

	// skip comments
	for (;;)
	{
		switch (c = *tokenp)
        {
		case ';':
			while ((c = *++tokenp) != '\0')
            {
				if (c == '\n')
					goto out;
			}
			/* GO THRU */
		case '\0':
			if (!fgets(inbuf, sizeof inbuf, in))
				return T_EOF;
			tokenp = inbuf;
			curline++;
			continue;
		case '\t':
		case ' ':
			tokenp++;
			continue;
		}
		break;
	}

out:
	lasttokenp = tokenp++;
	if (c == '\n')
		return T_EOL;

	if (isalpha(c) || c == '_')
    {
		cp = tp->name;
		ep = &tp->name[MAX_NAME - 1];
		*cp++ = (char)c;
		while (isalnum(*tokenp) || *tokenp == '_' || *tokenp == '+' || *tokenp == '-')
        {
			c = *tokenp++;
			if (cp < ep)
				*cp++ = (char)c;
		}
		*cp = '\0';

        // assume 'a', 'b', 'c', 'd', 'e' and 'f' is a hex value
        len = (int)(cp - tp->name);
        c = toupper(tp->name[0]);
        if (len == 1 && (c >= 'A' && c <= 'F'))
        {
			if ((e = tohex(tp->name, &tp->value)) != 0)
				return T_ERROR | e;

            return T_VALUE;
        }

		return T_NAME;
	}
    else if (isdigit(c))
    {
		cp = tp->name;
		ep = &tp->name[MAX_NAME - 1];
		*cp++ = (char)c;
		while (isxdigit(*tokenp))
        {
			c = *tokenp++;
			if (cp < ep)
				*cp++ = (char)c;
		}
		*cp = '\0';
		c = toupper(*tokenp);
		if (c == 'H')
        {
			tokenp++;
			if ((e = tohex(tp->name, &tp->value)) != 0)
				return T_ERROR | e;
		}
        else
        {
			if ((e = toint(tp->name, &tp->value)) != 0)
				return T_ERROR | e;
		}
		return T_VALUE;
	}
    else
		return T_CHAR | c;
}

/*
 *	UnGetToken
 */
void UnGetToken()
{
	tokenp = lasttokenp;
}

/*
 *	CancelLine
 */
void CancelLine()
{
	*tokenp = '\0';		// cancel this line
}

//
//  DEBUG
//

#if 0
int GetToken(TOKEN *tp)
{
    int     t;

    t = _GetToken(tp);
	printf("%d: ", curline);
	switch (t)
	{
	default:
		printf("0x%04x (%c)\n", t, t);
		break;
	case T_NAME:
		printf("T_NAME(%s)\n", tp->name);
		break;
	case T_VALUE:
		printf("T_VALUE: %d(0x%x)\n", tp->value, tp->value);
		break;
	case T_EOL:
		printf("T_EOL\n");
		break;
	case T_EOF:
		printf("T_EOF\n");
		break;
	}
	fflush(stdout);

    return t;
}
#endif

// vi: ts=4 sw=4

